@@ -19,7 +19,7 @@ import matplotlib.pyplot as plt
 from pathlib import Path
 from tqdm import tqdm
 
-from pgmpy.models import BayesianModel
+from pgmpy.models import BayesianModel, NaiveBayes
 from pgmpy.factors.discrete import TabularCPD
 from pgmpy.sampling import BayesianModelSampling  # GibbsSampling
 from pgmpy import readwrite
@@ -72,7 +72,7 @@ def to_bayesianmodel(model, verbose=3):
 
 
 # %% Make DAG
-def make_DAG(DAG, CPD=None, checkmodel=True, verbose=3):
+def make_DAG(DAG, CPD=None, methodtype='bayes', checkmodel=True, verbose=3):
     """Create Directed Acyclic Graph based on list.
 
     Parameters
@@ -81,33 +81,54 @@ def make_DAG(DAG, CPD=None, checkmodel=True, verbose=3):
         list containing source and target in the form of [('A','B'), ('B','C')].
     CPD : list, array-like
         Containing TabularCPD for each node.
+    methodtype : str (default: 'bayes')
+        * 'bayes': Bayesian model
+        * 'naivebayes': Special case of Bayesian Model where the only edges in the model are from the feature variables to the dependent variable. Or in other words, each tuple should start with the same variable name such as: edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
     checkmodel : bool
         Check the validity of the model. The default is True
     verbose : int, optional
         Print progress to screen. The default is 3.
         0: None, 1: ERROR, 2: WARN, 3: INFO (default), 4: DEBUG, 5: TRACE
 
-    Raises
-    ------
-    Exception
-        Should be list.
-
     Returns
     -------
-    pgmpy.models.BayesianModel.BayesianModel
-        model of the DAG.
+    dict keys:
+        * 'adjmat': Adjacency matrix
+        * 'model': pgmpy.models
+        * 'methodtype': methodtype
+        * 'model_edges': Edges
+
+    Examples
+    --------
+    >>> import bnlearn as bn
+    >>> edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+    >>> DAG = bn.make_DAG(edges, methodtype='naivebayes')
+    >>> bn.plot(DAG)
 
     """
     if (CPD is not None) and (not isinstance(CPD, list)):
         CPD=[CPD]
+
     if isinstance(DAG, dict):
         DAG = DAG.get('model', None)
+
     if (not isinstance(DAG, list)) and ('pgmpy' not in str(type(DAG))):
         raise Exception("[bnlearn] >Error: Input DAG should be a list. in the form [('A','B'), ('B','C')] or a <pgmpy.models.BayesianModel.BayesianModel>")
     elif ('pgmpy' in str(type(DAG))):
-        if verbose>=3: print('[bnlearn] >No changes made to existing Bayesian DAG.')
-    elif isinstance(DAG, list):
-        if verbose>=3: print('[bnlearn] >Bayesian DAG created.')
+        # Extract methodtype from existing model.
+        if ('bayesianmodel' in str(type(DAG)).lower()):
+            methodtype='bayes'
+        elif('naivebayes' in str(type(DAG)).lower()):
+            methodtype='naivebayes'
+        if verbose>=3: print('[bnlearn] >No changes made to existing %s DAG.' %(methodtype))
+    elif isinstance(DAG, list) and methodtype=='naivebayes':
+        if verbose>=3: print('[bnlearn] >%s DAG created.' %(methodtype))
+        edges=DAG
+        DAG = NaiveBayes()
+        DAG.add_edges_from(edges)
+        # modeel.add_nodes_from(DAG)
+    elif isinstance(DAG, list) and methodtype=='bayes':
+        if verbose>=3: print('[bnlearn] >%s DAG created.' %(methodtype))
         DAG = BayesianModel(DAG)
 
     if CPD is not None:
@@ -122,8 +143,9 @@ def make_DAG(DAG, CPD=None, checkmodel=True, verbose=3):
     out = {}
     out['adjmat'] = _dag2adjmat(DAG)
     out['model'] = DAG
+    out['methodtype'] = methodtype
+    out['model_edges'] = DAG.edges()
     return out
-    # return DAG
 
 
 # %% Print DAG
@@ -153,7 +175,7 @@ def print_CPD(DAG, checkmodel=False):
             # print CPDs using Maximum Likelihood Estimators
             for node in DAG.state_names:
                 print(DAG.estimate_cpd(node))
-        elif 'BayesianModel' in str(type(DAG)):
+        elif ('bayesianmodel' in str(type(DAG)).lower()) or ('naivebayes' in str(type(DAG)).lower()):
             # print CPDs using Bayesian Parameter Estimation
             if len(DAG.get_cpds())==0:
                 raise Exception('[bnlearn] >Error! This is a Bayesian DAG containing only edges, and no CPDs. Tip: you need to specify or learn the CPDs. Try: DAG=bn.parameter_learning.fit(DAG, df). At this point you can make a plot with: bn.plot(DAG).')
@@ -161,15 +183,15 @@ def print_CPD(DAG, checkmodel=False):
             for cpd in DAG.get_cpds():
                 print("CPD of {variable}:".format(variable=cpd.variable))
                 print(cpd)
-
-            print('[bnlearn] >Independencies:\n%s' %(DAG.get_independencies()))
+            if ('bayesianmodel' in str(type(DAG)).lower()):
+                print('[bnlearn] >Independencies:\n%s' %(DAG.get_independencies()))
             print('[bnlearn] >Nodes: %s' %(DAG.nodes()))
             print('[bnlearn] >Edges: %s' %(DAG.edges()))
 
         if checkmodel:
             _check_model(DAG, verbose=3)
     except:
-        print('[bnlearn] >No CPDs to print. Tip: use bnlearn.plot(DAG) to make a plot.')
+        print('[bnlearn] >No CPDs to print. Hint: Add CPDs as following: <bn.make_DAG(DAG, CPD=[cpd_A, cpd_B, etc])> and use bnlearn.plot(DAG) to make a plot.')
 
 
 # %%
@@ -506,9 +528,9 @@ def compare_networks(model_1, model_2, pos=None, showfig=True, figsize=(15, 8),
     Parameters
     ----------
     model_1 : dict
-        Results of model 1..
+        Results of model 1.
     model_2 : dict
-        Results of model 2..
+        Results of model 2.
     pos : graph, optional
         Coordinates of the network. If there are provided, the same structure will be used to plot the network.. The default is None.
     showfig : bool, optional
@@ -530,7 +552,7 @@ def compare_networks(model_1, model_2, pos=None, showfig=True, figsize=(15, 8),
     return(scores, adjmat_diff)
 
 
-# %% Set node properties
+# %% Get node properties
 def get_node_properties(model, node_color='#1f456e', node_size=None, verbose=3):
     # https://networkx.org/documentation/networkx-1.7/reference/generated/networkx.drawing.nx_pylab.draw_networkx_nodes.html
     nodes = {}
@@ -548,6 +570,65 @@ def get_node_properties(model, node_color='#1f456e', node_size=None, verbose=3):
     return nodes
 
 
+# %% Get node properties
+def get_edge_properties(model, color='#000000', weight=1, verbose=3):
+    """Collect edge properties.
+
+    Parameters
+    ----------
+    model : dict
+        dict containing (initialized) model.
+    color : str, (Default: '#000000')
+        The default color of the edges.
+    weight : float, (Default: 1)
+        The default weight of the edges. 
+    verbose : int, optional
+        Print progress to screen. The default is 3.
+        0: None, 1: ERROR, 2: WARN, 3: INFO (default), 4: DEBUG, 5: TRACE
+
+    Returns
+    -------
+    edges : dict.
+        Edge properties.
+    
+    Examples
+    --------
+    >>> # Example 1
+    >>> import bnlearn as bn
+    >>> edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+    >>> # Create DAG and store in model
+    >>> model = bn.make_DAG(edges)
+    >>> edge_properties = bn.get_edge_properties(model)
+    >>> # Adjust the properties
+    >>> edge_properties[('A', 'B')]['weight']=10
+    >>> edge_properties[('A', 'B')]['color']='#8A0707'
+    >>> # Make plot
+    >>> bn.plot(model, interactive=False, edge_properties=edge_properties)
+
+    """
+    # https://networkx.org/documentation/networkx-1.7/reference/generated/networkx.drawing.nx_pylab.draw_networkx_nodes.html
+    edges = {}
+    defaults = {'color':color, 'weight':weight}
+    adjmat = model.get('adjmat', None)
+    # Get model edges
+    model_edges = model['model'].edges() if (model.get('model_edges', None) is None) else model['model_edges']
+
+    # Store edge properties
+    if adjmat is not None:
+        if verbose>=3: print('[bnlearn]> Set edge properties.')
+        # For each edge, use the default properties.
+        for u, v in model_edges:
+            edge_property = defaults.copy()
+            # Use the edge weight from the adjmat
+            if not isinstance(adjmat.loc[u,v], np.bool_):
+                edge_property['weight']=adjmat.loc[u,v]
+            # Update edges dict
+            edges.update({(u,v): edge_property})
+
+    # Return dict with node properties
+    return edges
+
+
 # %% PLOT
 def plot(model,
          pos=None,
@@ -557,8 +638,9 @@ def plot(model,
          node_color=None,
          node_size=None,
          node_properties=None,
+         edge_properties=None,
          params_interactive={'height':'800px', 'width':'70%', 'notebook':False, 'layout':None, 'font_color': False, 'bgcolor':'#ffffff'},
-         params_static={'width':15, 'height':8, 'font_size':14, 'font_family':'sans-serif', 'alpha':0.8, 'node_shape':'o', 'layout':'fruchterman_reingold', 'font_color': '#000000', 'facecolor':'white'},
+         params_static={'width':15, 'height':8, 'font_size':14, 'font_family':'sans-serif', 'alpha':0.8, 'node_shape':'o', 'layout':'fruchterman_reingold', 'font_color': '#000000', 'facecolor':'white', 'edge_alpha':0.8, 'arrowstyle':'-|>', 'arrowsize':30},
          verbose=3):
     """Plot the learned stucture.
 
@@ -582,6 +664,11 @@ def plot(model,
     node_properties : dict (default: None).
         Dictionary containing custom node_color and node_size parameters for the network.
         The node properties can easily be retrieved using the function: node_properties = bn.get_node_properties(model)
+        node_properties = {'node1':{'node_color':'#8A0707','node_size':10},
+                           'node2':{'node_color':'#000000','node_size':30}}
+    edge_properties : dict (default: None).
+        Dictionary containing custom node_color and node_size parameters for the network.
+        The edge properties can easily be retrieved using the function: edge_properties = bn.get_edge_properties(model)
     params_interactive : dict.
         Dictionary containing various settings in case of creating interactive plots.
     params_static : dict.
@@ -632,17 +719,19 @@ def plot(model,
     # Plot properties
     defaults = {'height':'800px', 'width':'70%', 'notebook':False, 'layout':None, 'font_color': False, 'bgcolor':'#ffffff', 'directed':True}
     params_interactive = {**defaults, **params_interactive}
-    defaults = {'height':8, 'width':15, 'font_size':14, 'font_family':'sans-serif', 'alpha':0.8, 'layout':'fruchterman_reingold', 'font_color': 'k', 'facecolor':'#ffffff', 'node_shape':'o'}
+    defaults = {'height':8, 'width':15, 'font_size':14, 'font_family':'sans-serif', 'alpha':0.8, 'layout':'fruchterman_reingold', 'font_color': 'k', 'facecolor':'#ffffff', 'node_shape':'o', 'edge_alpha':0.8, 'arrowstyle':'-|>', 'arrowsize':30}
     params_static = {**defaults, **params_static}
     out = {}
     G = nx.DiGraph()  # Directed graph
     node_size_default = 10 if interactive else 800
     if (node_properties is not None) and (node_size is not None):
-        if verbose>=2: print('[bnlearn]> Warning: if both "node_size" and "node_properties" are used, "node_size" will dominate the results.')
+        if verbose>=2: print('[bnlearn]> Warning: if both "node_size" and "node_properties" are used, "node_size" will be used.')
 
     # Get node properties
     if node_properties is None:
-        node_properties = bnlearn.get_node_properties(model)
+        node_properties = bnlearn.get_node_properties(model, node_size=node_size_default)
+    if edge_properties is None:
+        edge_properties = bnlearn.get_edge_properties(model)
 
     # Set default node size based on interactive True/False
     for key in node_properties.keys():
@@ -656,8 +745,8 @@ def plot(model,
         bnmodel = model.copy()
 
     # Bayesian model
-    if 'BayesianModel' in str(type(bnmodel)) or 'pgmpy' in str(type(bnmodel)):
-        if verbose>=3: print('[bnlearn] >Plot based on BayesianModel')
+    if ('bayes' in str(type(bnmodel)).lower()) or ('pgmpy' in str(type(bnmodel)).lower()):
+        if verbose>=3: print('[bnlearn] >Plot based on Bayesian model')
         # positions for all nodes
         pos = bnlearn.network.graphlayout(bnmodel, pos=pos, scale=scale, layout=params_static['layout'], verbose=verbose)
     elif 'networkx' in str(type(bnmodel)):
@@ -671,30 +760,34 @@ def plot(model,
         pos = bnlearn.network.graphlayout(G, pos=pos, scale=scale, layout=params_static['layout'], verbose=verbose)
 
     # get node properties
-    nodelist, node_colors, node_sizes, edge_color, edge_weights = _plot_properties(G, bnmodel, node_properties, node_color, node_size)
+    nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights = _plot_properties(G, bnmodel, node_properties, edge_properties, node_color, node_size)
 
-    # Make interactive or static plot
+    # Plot
     if interactive:
-        _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, title=title, verbose=verbose)
+        # Make interactive plot
+        _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights, title, verbose=verbose)
     else:
-        _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, edge_color, edge_weights, pos)
+        # Make static plot
+        _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_colors, edge_weights)
 
     # Store
     out['pos']=pos
     out['G']=G
     out['node_properties']=node_properties
+    out['edge_properties']=edge_properties
     return(out)
 
 
 # %% Plot interactive
 # def _plot_static(model, params_static, nodelist, node_colors, node_sizes, title, verbose=3):
-def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, edge_color, edge_weights, pos):
+def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_color, edge_weights):
     # Bootup figure
     plt.figure(figsize=(params_static['width'], params_static['height']), facecolor=params_static['facecolor'])
     # nodes
     nx.draw_networkx_nodes(G, pos, nodelist=nodelist , node_size=node_sizes, alpha=params_static['alpha'], node_color=node_colors, node_shape=params_static['node_shape'])
     # edges
-    nx.draw_networkx_edges(G, pos, arrowstyle='->', edge_color=edge_color, width=edge_weights)
+    # nx.draw_networkx_edges(G, pos, arrowstyle='-|>', arrowsize=30, edge_color=edge_color, width=edge_weights)
+    nx.draw_networkx_edges(G, pos, arrowstyle=params_static['arrowstyle'], arrowsize=params_static['arrowsize'], edge_color=edge_color, width=edge_weights, alpha=params_static['edge_alpha'])
     # Labels
     nx.draw_networkx_labels(G, pos, font_size=params_static['font_size'], font_family=params_static['font_family'], font_color=params_static['font_color'])
     # Plot text of the weights
@@ -706,7 +799,7 @@ def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, edg
 
 
 # %% Plot interactive
-def _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, title, verbose=3):
+def _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights, title, verbose=3):
     try:
         from pyvis import network as net
         from IPython.core.display import display, HTML
@@ -718,11 +811,18 @@ def _plot_interactive(model, params_interactive, nodelist, node_colors, node_siz
     g = net.Network(**params_interactive)
     # Convert from graph G
     g.from_nx(G)
-    # g.edges
+    # Nodes
     for i,_ in enumerate(g.nodes):
         g.nodes[i]['color']=node_colors[np.where(nodelist==g.nodes[i].get('label'))[0][0]]
         g.nodes[i]['size']=node_sizes[np.where(nodelist==g.nodes[i].get('label'))[0][0]]
 
+    # Edges
+    g_edges = list(map(lambda x: (x.get('from'), x.get('to')), g.edges))
+    for i,_ in enumerate(g.edges):
+        idx = np.where(list(map(lambda x: g_edges[i]==x, edgelist)))[0][0]
+        g.edges[i]['color']=edge_colors[idx]
+        g.edges[i]['weight']=edge_weights[idx]
+
     # Create advanced buttons
     g.show_buttons(filter_=['physics'])
     # Display
@@ -733,16 +833,25 @@ def _plot_interactive(model, params_interactive, nodelist, node_colors, node_siz
 
 
 # %% Plot properties
-def _plot_properties(G, bnmodel, node_properties, node_color, node_size):
+def _plot_properties(G, bnmodel, node_properties, edge_properties, node_color, node_size):
     # Set edge properties in Graph G
     edges=[*bnmodel.edges()]
     for edge in edges:
-        G.add_edge(edge[0], edge[1], weight=1, color='#000000')
-    edge_color = [G[u][v].get('color') for u, v in G.edges()]
+        color = edge_properties.get((edge[0], edge[1])).get('color', '#000000')
+        weight = edge_properties.get((edge[0], edge[1])).get('weight', 1)
+        G.add_edge(edge[0], edge[1], weight=weight, color=color)
+        # arrowstyle = edge_properties.get((edge[0], edge[1])).get('arrowstyle', '-|>')
+        # arrowsize = edge_properties.get((edge[0], edge[1])).get('arrowsize', 30)
+        # G.add_edge(edge[0], edge[1], weight=weight, color=color, arrowstyle=arrowstyle, arrowsize=arrowsize)
+    
+    edgelist = list(G.edges())
+    edge_colors = [G[u][v].get('color') for u, v in G.edges()]
     edge_weights = [G[u][v].get('weight') for u, v in G.edges()]
+    # edge_arrowstyles = [G[u][v].get('arrowstyle') for u, v in G.edges()]
+    # edge_arrowsizes = [G[u][v].get('arrowsize') for u, v in G.edges()]
 
     # Node properties
-    nodelist = np.unique(edges)
+    nodelist = np.unique(edgelist)
     node_colors = []
     node_sizes = []
     for node in nodelist:
@@ -755,7 +864,7 @@ def _plot_properties(G, bnmodel, node_properties, node_color, node_size):
         else:
             node_sizes.append(node_properties[node].get('node_size'))
     # Return
-    return nodelist, node_colors, node_sizes, edge_color, edge_weights
+    return nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights
 
 
 # %%
@@ -828,12 +937,12 @@ def import_example(data='sprinkler', n=10000, verbose=3):
     ----------
     data : str, (default: sprinkler)
         Pre-defined examples.
-        'titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water'
+        'titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water', 'random'
     n : int, optional
         Number of samples to generate. The default is 1000.
     verbose : int, (default: 3)
         Print progress to screen.
-        0: None, 1: ERROR, 2: WARN, 3: INFO, 4: DEBUG, 5: TRACE
+        0: None, 1: Error, 2: Warning, 3: Info, 4: Debug, 5: Trace
 
     Returns
     -------
@@ -841,9 +950,12 @@ def import_example(data='sprinkler', n=10000, verbose=3):
 
     """
 
+    if data=='random':
+        return pd.DataFrame(np.random.randint(low=0, high=2, size=(n, 5)), columns=['A', 'B', 'C', 'D', 'E'])
+
     # Change name for downloading
     if data=='titanic': data = 'titanic_train'
-    
+
     # Download example dataset from github source
     PATH_TO_DATA = _download_example(data, verbose=verbose)
 
